<?php

namespace Drupal\lb_inspector\Batch\Operation;

use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\batch\Batch\Operation\EnumeratedOperationBase;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\lb_inspector\Inspector\LbInspectorPluginManagerAwareTrait;
use Drupal\lb_inspector\Service\Inspector;

/**
 * Batch operation for layout builder inspection.
 */
class InspectionOperation extends EnumeratedOperationBase {

  use EntityTypeManagerAwareTrait;
  use LbInspectorPluginManagerAwareTrait;

  /**
   * Constructs an InspectionOperation.
   *
   * @param array $ids
   *   The entity IDs.
   * @param string $entity_type_id
   *   The entity type to inspect.
   * @param string $field_name
   *   The layout builder field to inspect.
   * @param array $configuration
   *   Inspector  configuration.
   */
  public function __construct(array $ids, protected string $entity_type_id, protected string $field_name, protected array $configuration = []) {
    parent::__construct($ids);
  }

  /**
   * {@inheritdoc}
   */
  protected function init(array &$context): void {
    parent::init($context);
    $context['results']['lb_inspector'] = [
      'entity_type_id' => $this->entity_type_id,
      'field_name' => $this->field_name,
      'configuration' => $this->configuration,
      'rows' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function processItem(mixed $item, array &$context): void {
    // Load the entity.
    $entity = $this->getEntityTypeManager()
      ->getStorage($this->entity_type_id)
      ->load($item);
    if (!$entity) {
      return;
    }
    assert($entity instanceof FieldableEntityInterface);
    if (!$entity->hasField($this->field_name)) {
      return;
    }

    /** @var \Drupal\layout_builder\SectionListInterface $field_items */
    $field_items = $entity->get($this->field_name);
    $sections = $field_items->getSections();

    if (!empty($this->configuration['layouts'])) {
      $sections = Inspector::filterSectionsByLayout($sections, $this->configuration['layouts']);
    }

    // Loop through the sections, grab the components, filter to specified
    // regions.
    $components = [];
    foreach ($sections as $section) {
      $section_components = $section->getComponents();
      if (!empty($this->configuration['regions'])) {
        $section_components = Inspector::filterComponentsByRegion($section_components, $this->configuration['regions']);
      }
      $components = array_merge($components, $section_components);
    }

    // Run through the inspector.
    if (!empty($this->configuration['inspector'])) {
      $inspector_config = $this->configuration['inspector_config'] ?: [];
      /** @var \Drupal\lb_inspector\Inspector\LbInspectorInterface $inspector */
      $inspector = $this->getLbInspectorPluginManager()
        ->createInstance($this->configuration['inspector'], $inspector_config);
      $components = array_filter($components, [$inspector, 'inspectComponent']);
    }

    if (!empty(($components))) {
      $context['results']['lb_inspector']['rows'][] = $entity->id();
    }
  }

}
