<?php

declare(strict_types = 1);

namespace Drupal\lb_inspector\Controller;

use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\awareness\Pager\PagerManagerAwareTrait;
use Drupal\awareness\Session\CurrentUserAwareTrait;
use Drupal\awareness\User\UserDataAwareTrait;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\lb_inspector\Inspector\LbInspectorPluginManagerAwareTrait;

/**
 * Returns responses for Layout Builder Inspector routes.
 */
class LbInspectorController extends ControllerBase {

  use CurrentUserAwareTrait;
  use EntityTypeManagerAwareTrait;
  use LbInspectorPluginManagerAwareTrait;
  use PagerManagerAwareTrait;
  use UserDataAwareTrait;

  /**
   * Displays inspection results.
   */
  public function __invoke(): array {
    $build = [];

    $results = $this->getUserData()
      ->get('lb_inspector', $this->getCurrentUser()->id(), 'inspection_result');

    $build['table'] = [
      '#theme' => 'table',
      '#header' => [$this->t('ID'), $this->t('Label')],
      '#rows' => [],
      '#empty' => $this->t('No matching layout builder content.'),
    ];

    if (empty($results['lb_inspector'])) {
      $build['table']['#empty'] = $this->t('No inspection ran.');
      return $build;
    }

    $build['inspection'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Inspection'),
      '#weight' => -10,
    ];

    $build['inspection']['entity_type'] = [
      '#type' => 'item',
      '#title' => $this->t('Entity type'),
      '#plain_text' => $results['lb_inspector']['entity_type_id'],
    ];

    $build['inspection']['field_name'] = [
      '#type' => 'item',
      '#title' => $this->t('Field name'),
      '#plain_text' => $results['lb_inspector']['field_name'],
    ];

    if (!empty($results['lb_inspector']['configuration']['bundles'])) {
      $build['inspection']['bundles'] = [
        '#type' => 'item',
        '#title' => $this->t('Bundles'),
        '#plain_text' => implode(', ', $results['lb_inspector']['configuration']['bundles']),
      ];
    }

    if (!empty($results['lb_inspector']['configuration']['layouts'])) {
      $build['inspection']['layouts'] = [
        '#type' => 'item',
        '#title' => $this->t('Layouts'),
        '#plain_text' => implode(', ', $results['lb_inspector']['configuration']['layouts']),
      ];
    }

    if (!empty($results['lb_inspector']['configuration']['regions'])) {
      $build['inspection']['regions'] = [
        '#type' => 'item',
        '#title' => $this->t('Regions'),
        '#plain_text' => implode(', ', $results['lb_inspector']['configuration']['regions']),
      ];
    }

    if (!empty($results['lb_inspector']['configuration']['inspector'])) {
      $build['inspection']['inspector'] = [
        '#type' => 'item',
        '#title' => $this->t('Inspector'),
        '#plain_text' => $results['lb_inspector']['configuration']['inspector'],
      ];

      $inspector_config = $results['lb_inspector']['configuration']['inspector_config'] ?: [];
      /** @var \Drupal\lb_inspector\Inspector\LbInspectorInterface $inspector */
      $inspector = $this->getLbInspectorPluginManager()
        ->createInstance($results['lb_inspector']['configuration']['inspector'], $inspector_config);

      $build['inspection']['inspector_config'] = $inspector->configurationSummary();
    }

    // Instantiate pager, trim IDs to desired page, load entities.
    $pager = $this->getPagerManager()
      ->createPager(count($results['lb_inspector']['rows']), 50);
    $ids = array_slice($results['lb_inspector']['rows'], $pager->getCurrentPage() * 50, 50);
    $entities = $this->getEntityTypeManager()
      ->getStorage($results['lb_inspector']['entity_type_id'])
      ->loadMultiple($ids);

    // Create tables rows.
    $rows = array_map(function ($entity) {
      return [
        $entity->id(),
        Link::fromTextAndUrl($entity->label(), $entity->toUrl()),
      ];
    }, $entities);
    $build['table']['#rows'] = $rows;

    $build['pager'] = ['#type' => 'pager'];

    return $build;
  }

}
