<?php

declare(strict_types = 1);

namespace Drupal\lb_inspector\Form;

use Drupal\awareness\Entity\EntityTypeBundleInfoAwareTrait;
use Drupal\awareness\Layout\LayoutPluginManagerAwareTrait;
use Drupal\batch\Batch\BatchBuilder;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\lb_inspector\Batch\Finish\InspectionFinish;
use Drupal\lb_inspector\Batch\Operation\InspectionOperation;
use Drupal\lb_inspector\Inspector\LbInspectorPluginManagerAwareTrait;
use Drupal\lb_inspector\Service\LbInspectorInspectorAwareTrait;

/**
 * Provides a Layout Builder Inspector form.
 */
class LbInspectorForm extends FormBase {

  use EntityTypeBundleInfoAwareTrait;
  use LayoutPluginManagerAwareTrait;
  use LbInspectorInspectorAwareTrait;
  use LbInspectorPluginManagerAwareTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'lb_inspector_inspect';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $lb_entity_types = $this->getLbInspector()->getLayoutBuilderEntityTypes();
    $entity_type_options = array_map(function ($type) {
      return $type->getLabel();
    }, $lb_entity_types);

    $entity_type_default = NULL;
    if (count($entity_type_options) == 1) {
      $keys = array_keys($entity_type_options);
      $entity_type_default = reset($keys);
    }

    $content_section_id = Html::getUniqueId($this->getFormId() . '_content');

    $form['content'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Layout'),
      '#id' => $content_section_id,
    ];

    $form['content']['entity_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#options' => $entity_type_options,
      '#default_value' => $entity_type_default,
      '#required' => TRUE,
      '#ajax' => [
        'callback' => [static::class, 'ajaxContent'],
        'wrapper' => $content_section_id,
      ],
    ];

    if (!$selected_entity_type = $form_state->getValue('entity_type')) {
      $selected_entity_type = $entity_type_default;
    }

    if ($selected_entity_type) {
      $field_options = $this->getLbInspector()
        ->getLayoutBuilderEntityFields($selected_entity_type);
      $field_options = array_combine($field_options, $field_options);

      $field_default = NULL;
      if (count($field_options) == 1) {
        $keys = array_keys($field_options);
        $field_default = reset($keys);
      }

      $form['content']['field'] = [
        '#type' => 'select',
        '#title' => $this->t('Field'),
        '#description' => $this->t('Choose which layout builder field to inspect.'),
        '#options' => $field_options,
        '#required' => TRUE,
        '#default_value' => $field_default,
      ];

      $bundle_options = array_map(function ($bundle) {
        return $bundle['label'];
      }, $this->getEntityTypeBundleInfo()->getBundleInfo($selected_entity_type));
      asort($bundle_options);

      if (count($bundle_options) > 1) {
        $form['content']['bundles'] = [
          '#type' => 'checkboxes',
          '#title' => $this->t('Bundle'),
          '#description' => $this->t('Restrict inspection to only these bundles. Selecting no bundles will inspect all bundles.'),
          '#options' => $bundle_options,
        ];
      }
    }

    $layouts = $this->getLayoutPluginManager()->getDefinitions();
    $layout_options = array_combine(array_keys($layouts), array_keys($layouts));
    asort($layout_options);

    $layout_section_id = Html::getUniqueId($this->getFormId() . '_layouts');

    $form['layouts'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Layouts'),
      '#id' => $layout_section_id,
    ];

    $selected_layouts = array_filter($form_state->getValue('layouts') ?: []);

    $form['layouts']['layouts'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Layouts'),
      '#description' => $this->t('Restrict inspection to only these layouts. Selecting no layouts will inspect all layouts.'),
      '#options' => $layout_options,
      '#ajax' => [
        'callback' => [static::class, 'ajaxLayouts'],
        'wrapper' => $layout_section_id,
      ],
    ];

    $region_options = [];
    foreach ($layouts as $layout_id => $layout) {
      if (empty($selected_layouts) || in_array($layout_id, array_keys($selected_layouts))) {
        $region_options = array_merge($region_options, array_keys($layout->getRegions()));
      }
    }
    $region_options = array_combine($region_options, $region_options);
    asort($region_options);

    $form['layouts']['regions'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Regions'),
      '#description' => $this->t('Restrict inspection to only these layout regions. Selecting no regions will inspect all regions.'),
      '#options' => $region_options,
    ];

    $inspector_section_id = Html::getUniqueId($this->getFormId() . '_inspector');

    $form['inspector'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Inspector'),
      '#id' => $inspector_section_id,
    ];

    $inspector_options = array_map(function ($definition) {
      return $definition['label'];
    }, $this->getLbInspectorPluginManager()->getDefinitions());

    $form['inspector']['inspector'] = [
      '#type' => 'select',
      '#options' => $inspector_options,
      '#ajax' => [
        'callback' => [static::class, 'ajaxInspector'],
        'wrapper' => $inspector_section_id,
      ],
      '#empty_option' => $this->t('None'),
    ];

    // Inspector plugin form.

    $form['inspector']['inspector_config'] = [
      '#tree' => TRUE,
    ];

    if ($inspector_plugin_id = $form_state->getValue('inspector')) {
      $inspector_config = $form_state->getValue('inspector_config') ?: [];
      /** @var \Drupal\lb_inspector\Inspector\LbInspectorInterface $inspector */
      $inspector = $this->getLbInspectorPluginManager()
        ->createInstance($inspector_plugin_id, $inspector_config);
      $form['inspector']['inspector_config'] = $inspector->buildConfigurationForm($form['inspector']['inspector_config'], $form_state);
    }

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Inspect'),
      ],
    ];

    return $form;
  }

  /**
   * Ajax callback for handling updates in the content section.
   */
  public static function ajaxContent(array $form, FormStateInterface $form_state) {
    return $form['content'];
  }

  /**
   * Ajax callback for handling updates in the layouts section.
   */
  public static function ajaxLayouts(array $form, FormStateInterface $form_state) {
    return $form['layouts'];
  }

  /**
   * Ajax callback for handling updates in the inspector section.
   */
  public static function ajaxInspector(array $form, FormStateInterface $form_state) {
    return $form['inspector'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $entity_type_id = $form_state->getValue('entity_type');
    $field = $form_state->getValue('field');
    $bundles = array_filter($form_state->getValue('bundles') ?: []);

    $configuration = [
      'bundles' => $bundles,
      'layouts' =>  array_filter($form_state->getValue('layouts') ?: []),
      'regions' => array_filter($form_state->getValue('regions') ?: []),
      'inspector' => $form_state->getValue('inspector'),
      'inspector_config' => $form_state->getValue('inspector_config'),
    ];

    $ids = $this->getLbInspector()->queryEntities($entity_type_id, $field, $bundles);
    $operation = new InspectionOperation($ids, $entity_type_id, $field, $configuration);
    $batch = new BatchBuilder();
    $batch->addBatchOperation($operation);
    $finish = new InspectionFinish();
    $batch->setFinishOperation($finish);
    batch_set($batch->toArray());
  }

}
