<?php

declare(strict_types = 1);

namespace Drupal\lb_inspector\Plugin\lb_inspector\inspector;

use Drupal\awareness\Entity\EntityTypeManagerAwareTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder\SectionComponent;
use Drupal\lb_inspector\Inspector\LbInspectorPluginBase;

/**
 * Plugin implementation of the lb_inspector.
 *
 * @LbInspector(
 *   id = "inline_block_content",
 *   label = @Translation("Inline Block Content"),
 *   description = @Translation("Inspect block content in layout builder
 *   content.")
 * )
 */
class InlineBlockContent extends LbInspectorPluginBase {

  use EntityTypeManagerAwareTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function inspectComponent(SectionComponent $component): bool {
    $id = explode(':', $component->getPluginId());
    if ($id[0] != 'inline_block') {
      return FALSE;
    }

    $configuration = $component->get('configuration');

    // Query matching revision.
    $query = $this->getEntityQuery('block_content');
    $query->condition('revision_id', $configuration['block_revision_id']);

    foreach ($this->configuration['conditions'] as $condition) {
      if (empty(trim($condition['field']))) {
        continue;
      }
      $query->condition($condition['field'], $condition['value'], $condition['op']);
    }

    $result = $query->accessCheck(FALSE)->allRevisions()->execute();
    return !empty($result);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'conditions' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $op_options = [
      '=',
      '<>',
      '>',
      '>=',
      '<',
      '<=',
      'STARTS_WITH',
      'CONTAINS',
      'ENDS_WITH',
      'IS NULL',
      'IS NOT NULL',
    ];
    $op_options = array_combine($op_options, $op_options);

    $form['conditions']['#after_build'][] = [$this, 'buildTable'];

    foreach ([0, 1, 2, 3, 4, 5] as $i) {
      $form['conditions'][$i]['field'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Field'),
      ];

      $form['conditions'][$i]['op'] = [
        '#type' => 'select',
        '#options' => $op_options,
        '#title' => $this->t('Operator'),
      ];

      $form['conditions'][$i]['value'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Value'),
      ];
    }

    return $form;
  }

  /**
   * After build callback for the conditions table.
   */
  public function buildTable($element) {
    $rows = [];
    foreach ([0, 1, 2, 3, 4, 5] as $i) {
      $row = [];
      foreach (['field', 'op', 'value'] as $col) {
        $element[$i][$col]['#title_display'] = 'hidden';
        $row[$col] = ['data' => $element[$i][$col]];
        unset($element[$i][$col]);
      }
      $rows[] = $row;
    }

    $element['table'] = [
      '#theme' => 'table',
      '#header' => [$this->t('Field'), $this->t('Operator'), $this->t('Value')],
      '#rows' => $rows,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationSummary(): array {
    $items = [];
    foreach ($this->getConfiguration()['conditions'] as $condition) {
      if (!empty($condition['field'])) {
        $items[] = "{$condition['field']} {$condition['op']} {$condition['value']}";
      }
    }
    return [
      '#theme' => 'item_list',
      '#items' => $items,
    ];
  }

}
